package com.arunta.app.remote.example;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.rmi.RemoteException;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.core.LoggerContext;

import com.arunta.app.bom.Recording;
import com.arunta.app.bom.SecurityKey;
import com.arunta.app.remote.TRMonitorServiceRI;
import com.arunta.app.remote.TRRemoteUtility;
import com.arunta.app.remote.TRSystemServiceRI;
import com.arunta.base.audio.streaming.StreamPacketReceiver;
import com.arunta.base.exception.MonitoringDisabledException;
import com.arunta.base.exception.SecurityException;
import com.arunta.base.exception.TooManySessionsException;

public class TRMonitoringExample {

    public static void main(String[] args) {
        
        LoggerContext loggerContext = (LoggerContext) LogManager.getContext(false);
        try {
            loggerContext.setConfigLocation(new URI("log4j2.xml"));
        }
        catch (URISyntaxException e) {
            e.printStackTrace();
            System.exit(-1);
        }
        
        // --
        // Connect and get a security key
        // --
        
        // From the Remote Manager configuration ...
        final String rmIpAddress = "192.168.4.115";
        final int rmPort = 10010;
        final String trPassword = "0000";

        TRSystemServiceRI trSystemService = null;
        try {
            trSystemService = TRRemoteUtility.getSystemService(rmIpAddress, rmPort);;
        } 
        catch (RemoteException e) {
            e.printStackTrace();
            System.exit(1);
        } 

        SecurityKey securityKey = null;
        try {
            securityKey = trSystemService.logon(trPassword, "example");
        } 
        catch (RemoteException e) {
            e.printStackTrace();
            System.exit(1);
        } 
        catch (SecurityException e) {
            e.printStackTrace();
            System.exit(1);
        }

        // --
        // Obtain reference to the remote interface for the monitoring service
        // --
        
        TRMonitorServiceRI trMonitorService = null;
        try {
            trMonitorService = TRRemoteUtility.getMonitorService(rmIpAddress, rmPort);
        }
        catch (RemoteException e) {
            e.printStackTrace();
            System.exit(1);
        } 

        // --
        // Start a Monitoring Packet Receiver
        // --
        
        final String localIpAddress = "192.168.4.212";
        final int localPort = 10099;

        InetSocketAddress socketAddress = new InetSocketAddress(localIpAddress, localPort);
        
        StreamPacketReceiver mpr = new StreamPacketReceiver(socketAddress);
        mpr.setPacketListener(new TRMonitoringClient());
        try {
            mpr.startProcessing();
        } 
        catch (IOException e) {
            e.printStackTrace();
            System.exit(1);
        }

        // --
        // Instruct the TR to start sending packets for an active call
        // --
        
        List activeRecordingList = null;
        try {
            activeRecordingList = trMonitorService.getActiveRecordings(securityKey.toString());
        } 
        catch (RemoteException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        } 
        catch (SecurityException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        }
        
        if (activeRecordingList.isEmpty()) {
            System.out.println("No active calls to monitor.");
            mpr.stopProcessing();
            System.exit(1);
        }

        Recording rec = (Recording) activeRecordingList.get(0); // represents the  most recent call
        
        try {
            trMonitorService.startRemoteMonitor(securityKey.toString(), rec.getChannelNumber(), socketAddress);
        } 
        catch (RemoteException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        } 
        catch (TooManySessionsException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        } 
        catch (MonitoringDisabledException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        }
        catch (SecurityException e) {
            mpr.stopProcessing();
            e.printStackTrace();
            System.exit(1);
        }
        
        // --
        // At this point the TR should be sending packets to this client.
        // --
        
        // --
        // Ctrl-C stops the show.
        // --
        final Object end = new Object();
        Runtime.getRuntime().addShutdownHook(new Thread() {
           public void run() {
               end.notify();
           }
        });
        
        synchronized(end) {
            try {
                end.wait();
            } 
            catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        // --
        // Time to end monitoring
        // --
        
        try {
            trMonitorService.stopRemoteMonitor(securityKey.toString(), rec.getChannelNumber(), socketAddress);
        } 
        catch (RemoteException e) {
            e.printStackTrace();
        } 
        catch (SecurityException e) {
            e.printStackTrace();
        }

        mpr.stopProcessing();
    }

}
